﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Data.Linq;
using System.Linq;
using System.ServiceModel;
using HeuristicLab.Services.OKB.DataAccess;

namespace HeuristicLab.Services.OKB {
  /// <summary>
  /// Implementation of the OKB query service (interface <see cref="IQueryService"/>).
  /// </summary>
  [ServiceBehavior(IncludeExceptionDetailInFaults = true)]
  public class QueryService : IQueryService {
    public IEnumerable<DataTransfer.Filter> GetFilters() {
      List<DataTransfer.Filter> filters = new List<DataTransfer.Filter>();
      using (OKBDataContext okb = new OKBDataContext()) {
        //DataLoadOptions dlo = new DataLoadOptions();
        //dlo.LoadWith<Algorithm>(x => x.AlgorithmData);
        //dlo.LoadWith<AlgorithmData>(x => x.DataType);
        //dlo.LoadWith<Problem>(x => x.ProblemData);
        //dlo.LoadWith<ProblemData>(x => x.DataType);
        //dlo.LoadWith<AlgorithmParameter>(x => x.DataType);
        //dlo.LoadWith<ProblemParameter>(x => x.DataType);
        //dlo.LoadWith<Result>(x => x.DataType);
        //okb.LoadOptions = dlo;

        // run filters
        filters.Add(new DataTransfer.OrdinalComparisonLongFilter(typeof(RunRandomSeedFilter).AssemblyQualifiedName, "Run Random Seed"));
        filters.Add(new DataTransfer.OrdinalComparisonDateTimeFilter(typeof(RunCreatedDateFilter).AssemblyQualifiedName, "Run Created Date"));
        filters.Add(new DataTransfer.SetComparisonStringFilter(typeof(RunUserNameFilter).AssemblyQualifiedName, "Run User Name"));
        filters.Add(new DataTransfer.SetComparisonStringFilter(typeof(RunClientNameFilter).AssemblyQualifiedName, "Run Client Name"));

        // result filters
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(ResultNameFilter).AssemblyQualifiedName, "Result Name", okb.Results.Select(x => x.Name).Distinct().ToArray()));
        foreach (Result entity in okb.Results.Where(x => x.DataType.SqlName == "varbinary").AsEnumerable().Distinct<Result>(new GenericEqualityComparer<Result>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameSetComparisonStringAvailableValuesFilter(typeof(ResultBlobValueDataTypeNameFilter).AssemblyQualifiedName, "Result " + entity.Name + " Value Data Type Name", entity.Name, okb.ResultBlobValues.Where(x => x.Result.Name == entity.Name).Select(x => x.DataType.Name).Distinct().ToArray()));
        foreach (Result entity in okb.Results.Where(x => x.DataType.SqlName == "varbinary").AsEnumerable().Distinct<Result>(new GenericEqualityComparer<Result>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameEqualityComparisonByteArrayFilter(typeof(ResultBlobValueValueFilter).AssemblyQualifiedName, "Result " + entity.Name + " Value", entity.Name));
        foreach (Result entity in okb.Results.Where(x => x.DataType.SqlName == "bit").AsEnumerable().Distinct<Result>(new GenericEqualityComparer<Result>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameEqualityComparisonBoolFilter(typeof(ResultBoolValueValueFilter).AssemblyQualifiedName, "Result " + entity.Name + " Value", entity.Name));
        foreach (Result entity in okb.Results.Where(x => x.DataType.SqlName == "float").AsEnumerable().Distinct<Result>(new GenericEqualityComparer<Result>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameOrdinalComparisonDoubleFilter(typeof(ResultFloatValueValueFilter).AssemblyQualifiedName, "Result " + entity.Name + " Value", entity.Name));
        foreach (Result entity in okb.Results.Where(x => x.DataType.SqlName == "bigint").AsEnumerable().Distinct<Result>(new GenericEqualityComparer<Result>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameOrdinalComparisonLongFilter(typeof(ResultIntValueValueFilter).AssemblyQualifiedName, "Result " + entity.Name + " Value", entity.Name));
        foreach (Result entity in okb.Results.Where(x => x.DataType.SqlName == "nvarchar").AsEnumerable().Distinct<Result>(new GenericEqualityComparer<Result>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameSetComparisonStringFilter(typeof(ResultStringValueValueFilter).AssemblyQualifiedName, "Result " + entity.Name + " Value", entity.Name));

        // algorithm parameter filters
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(AlgorithmParameterNameFilter).AssemblyQualifiedName, "Algorithm Parameter Name", okb.AlgorithmParameters.Select(x => x.Name).Distinct().ToArray()));
        foreach (AlgorithmParameter entity in okb.AlgorithmParameters.Where(x => x.DataType.SqlName == "varbinary").AsEnumerable().Distinct<AlgorithmParameter>(new GenericEqualityComparer<AlgorithmParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameSetComparisonStringAvailableValuesFilter(typeof(AlgorithmParameterBlobValueDataTypeNameFilter).AssemblyQualifiedName, "Algorithm Parameter " + entity.Name + " Value Data Type Name", entity.Name, okb.AlgorithmParameterBlobValues.Where(x => x.AlgorithmParameter.Name == entity.Name).Select(x => x.DataType.Name).Distinct().ToArray()));
        foreach (AlgorithmParameter entity in okb.AlgorithmParameters.Where(x => x.DataType.SqlName == "varbinary").AsEnumerable().Distinct<AlgorithmParameter>(new GenericEqualityComparer<AlgorithmParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameEqualityComparisonByteArrayFilter(typeof(AlgorithmParameterBlobValueValueFilter).AssemblyQualifiedName, "Algorithm Parameter " + entity.Name + " Value", entity.Name));
        foreach (AlgorithmParameter entity in okb.AlgorithmParameters.Where(x => x.DataType.SqlName == "bit").AsEnumerable().Distinct<AlgorithmParameter>(new GenericEqualityComparer<AlgorithmParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameEqualityComparisonBoolFilter(typeof(AlgorithmParameterBoolValueValueFilter).AssemblyQualifiedName, "Algorithm Parameter " + entity.Name + " Value", entity.Name));
        foreach (AlgorithmParameter entity in okb.AlgorithmParameters.Where(x => x.DataType.SqlName == "float").AsEnumerable().Distinct<AlgorithmParameter>(new GenericEqualityComparer<AlgorithmParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameOrdinalComparisonDoubleFilter(typeof(AlgorithmParameterFloatValueValueFilter).AssemblyQualifiedName, "Algorithm Parameter " + entity.Name + " Value", entity.Name));
        foreach (AlgorithmParameter entity in okb.AlgorithmParameters.Where(x => x.DataType.SqlName == "bigint").AsEnumerable().Distinct<AlgorithmParameter>(new GenericEqualityComparer<AlgorithmParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameOrdinalComparisonLongFilter(typeof(AlgorithmParameterIntValueValueFilter).AssemblyQualifiedName, "Algorithm Parameter " + entity.Name + " Value", entity.Name));
        foreach (AlgorithmParameter entity in okb.AlgorithmParameters.Where(x => x.DataType.SqlName == "nvarchar").AsEnumerable().Distinct<AlgorithmParameter>(new GenericEqualityComparer<AlgorithmParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameSetComparisonStringFilter(typeof(AlgorithmParameterStringValueValueFilter).AssemblyQualifiedName, "Algorithm Parameter " + entity.Name + " Value", entity.Name));

        // algorithm filters
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(AlgorithmNameFilter).AssemblyQualifiedName, "Algorithm Name", okb.Algorithms.Select(x => x.Name).Distinct().ToArray()));
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(AlgorithmClassNameFilter).AssemblyQualifiedName, "Algorithm Class Name", okb.AlgorithmClasses.Select(x => x.Name).Distinct().ToArray()));
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(AlgorithmPlatformNameFilter).AssemblyQualifiedName, "Algorithm Platform Name", okb.Platforms.Select(x => x.Name).Distinct().ToArray()));
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(AlgorithmDataDataTypeNameFilter).AssemblyQualifiedName, "Algorithm Data Type Name", okb.Algorithms.Select(x => x.AlgorithmData.DataType.Name).Distinct().ToArray()));

        // problem parameter filters
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(ProblemParameterNameFilter).AssemblyQualifiedName, "Problem Parameter Name", okb.ProblemParameters.Select(x => x.Name).Distinct().ToArray()));
        foreach (ProblemParameter entity in okb.ProblemParameters.Where(x => x.DataType.SqlName == "varbinary").AsEnumerable().Distinct<ProblemParameter>(new GenericEqualityComparer<ProblemParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameSetComparisonStringAvailableValuesFilter(typeof(ProblemParameterBlobValueDataTypeNameFilter).AssemblyQualifiedName, "Problem Parameter " + entity.Name + " Value Data Type Name", entity.Name, okb.ProblemParameterBlobValues.Where(x => x.ProblemParameter.Name == entity.Name).Select(x => x.DataType.Name).Distinct().ToArray()));
        foreach (ProblemParameter entity in okb.ProblemParameters.Where(x => x.DataType.SqlName == "varbinary").AsEnumerable().Distinct<ProblemParameter>(new GenericEqualityComparer<ProblemParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameEqualityComparisonByteArrayFilter(typeof(ProblemParameterBlobValueValueFilter).AssemblyQualifiedName, "Problem Parameter " + entity.Name + " Value", entity.Name));
        foreach (ProblemParameter entity in okb.ProblemParameters.Where(x => x.DataType.SqlName == "bit").AsEnumerable().Distinct<ProblemParameter>(new GenericEqualityComparer<ProblemParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameEqualityComparisonBoolFilter(typeof(ProblemParameterBoolValueValueFilter).AssemblyQualifiedName, "Problem Parameter " + entity.Name + " Value", entity.Name));
        foreach (ProblemParameter entity in okb.ProblemParameters.Where(x => x.DataType.SqlName == "float").AsEnumerable().Distinct<ProblemParameter>(new GenericEqualityComparer<ProblemParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameOrdinalComparisonDoubleFilter(typeof(ProblemParameterFloatValueValueFilter).AssemblyQualifiedName, "Problem Parameter " + entity.Name + " Value", entity.Name));
        foreach (ProblemParameter entity in okb.ProblemParameters.Where(x => x.DataType.SqlName == "bigint").AsEnumerable().Distinct<ProblemParameter>(new GenericEqualityComparer<ProblemParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameOrdinalComparisonLongFilter(typeof(ProblemParameterIntValueValueFilter).AssemblyQualifiedName, "Problem Parameter " + entity.Name + " Value", entity.Name));
        foreach (ProblemParameter entity in okb.ProblemParameters.Where(x => x.DataType.SqlName == "nvarchar").AsEnumerable().Distinct<ProblemParameter>(new GenericEqualityComparer<ProblemParameter>((x, y) => x.Name == y.Name)))
          filters.Add(new DataTransfer.NameSetComparisonStringFilter(typeof(ProblemParameterStringValueValueFilter).AssemblyQualifiedName, "Problem Parameter " + entity.Name + " Value", entity.Name));

        // problem filters
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(ProblemNameFilter).AssemblyQualifiedName, "Problem Name", okb.Problems.Select(x => x.Name).Distinct().ToArray()));
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(ProblemClassNameFilter).AssemblyQualifiedName, "Problem Class Name", okb.ProblemClasses.Select(x => x.Name).Distinct().ToArray()));
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(ProblemPlatformNameFilter).AssemblyQualifiedName, "Problem Platform Name", okb.Platforms.Select(x => x.Name).Distinct().ToArray()));
        filters.Add(new DataTransfer.SetComparisonStringAvailableValuesFilter(typeof(ProblemDataDataTypeNameFilter).AssemblyQualifiedName, "Problem Data Type Name", okb.Problems.Select(x => x.ProblemData.DataType.Name).Distinct().ToArray()));

        // and/or filters
        filters.Add(new DataTransfer.AndFilter(typeof(AndFilter).AssemblyQualifiedName, "And"));
        filters.Add(new DataTransfer.OrFilter(typeof(OrFilter).AssemblyQualifiedName, "Or"));
      }
      return filters.OrderBy(x => x.Label);
    }
    public long QueryNumberOfRuns(DataTransfer.Filter filter) {
      using (OKBDataContext okb = new OKBDataContext()) {
        IFilter convertedFilter = (IFilter)Activator.CreateInstance(Type.GetType(filter.FilterTypeName), filter);
        return okb.Runs.Where(convertedFilter.Expression).LongCount();
      }
    }
    public IEnumerable<DataTransfer.Run> QueryRuns(DataTransfer.Filter filter) {
      using (OKBDataContext okb = new OKBDataContext()) {
        //DataLoadOptions dlo = new DataLoadOptions();
        //dlo.LoadWith<Run>(x => x.ResultBlobValues);
        //dlo.LoadWith<Run>(x => x.ResultBoolValues);
        //dlo.LoadWith<Run>(x => x.ResultFloatValues);
        //dlo.LoadWith<Run>(x => x.ResultIntValues);
        //dlo.LoadWith<Run>(x => x.ResultStringValues);
        //dlo.LoadWith<Run>(x => x.Experiment);
        //dlo.LoadWith<ResultBlobValue>(x => x.Result);
        //dlo.LoadWith<ResultBlobValue>(x => x.DataType);
        //dlo.LoadWith<ResultBoolValue>(x => x.Result);
        //dlo.LoadWith<ResultBoolValue>(x => x.DataType);
        //dlo.LoadWith<ResultFloatValue>(x => x.Result);
        //dlo.LoadWith<ResultFloatValue>(x => x.DataType);
        //dlo.LoadWith<ResultIntValue>(x => x.Result);
        //dlo.LoadWith<ResultIntValue>(x => x.DataType);
        //dlo.LoadWith<ResultStringValue>(x => x.Result);
        //dlo.LoadWith<ResultStringValue>(x => x.DataType);
        //dlo.LoadWith<Result>(x => x.Algorithm);
        //dlo.LoadWith<Result>(x => x.DataType);
        //dlo.LoadWith<Experiment>(x => x.Algorithm);
        //dlo.LoadWith<Experiment>(x => x.Problem);
        //dlo.LoadWith<Experiment>(x => x.AlgorithmParameterBlobValues);
        //dlo.LoadWith<Experiment>(x => x.AlgorithmParameterBoolValues);
        //dlo.LoadWith<Experiment>(x => x.AlgorithmParameterFloatValues);
        //dlo.LoadWith<Experiment>(x => x.AlgorithmParameterIntValues);
        //dlo.LoadWith<Experiment>(x => x.AlgorithmParameterStringValues);
        //dlo.LoadWith<Experiment>(x => x.ProblemParameterBlobValues);
        //dlo.LoadWith<Experiment>(x => x.ProblemParameterBoolValues);
        //dlo.LoadWith<Experiment>(x => x.ProblemParameterFloatValues);
        //dlo.LoadWith<Experiment>(x => x.ProblemParameterIntValues);
        //dlo.LoadWith<Experiment>(x => x.ProblemParameterStringValues);
        //dlo.LoadWith<AlgorithmParameterBlobValue>(x => x.AlgorithmParameter);
        //dlo.LoadWith<AlgorithmParameterBlobValue>(x => x.DataType);
        //dlo.LoadWith<AlgorithmParameterBoolValue>(x => x.AlgorithmParameter);
        //dlo.LoadWith<AlgorithmParameterBoolValue>(x => x.DataType);
        //dlo.LoadWith<AlgorithmParameterFloatValue>(x => x.AlgorithmParameter);
        //dlo.LoadWith<AlgorithmParameterFloatValue>(x => x.DataType);
        //dlo.LoadWith<AlgorithmParameterIntValue>(x => x.AlgorithmParameter);
        //dlo.LoadWith<AlgorithmParameterIntValue>(x => x.DataType);
        //dlo.LoadWith<AlgorithmParameterStringValue>(x => x.AlgorithmParameter);
        //dlo.LoadWith<AlgorithmParameterStringValue>(x => x.DataType);
        //dlo.LoadWith<ProblemParameterBlobValue>(x => x.ProblemParameter);
        //dlo.LoadWith<ProblemParameterBlobValue>(x => x.DataType);
        //dlo.LoadWith<ProblemParameterBoolValue>(x => x.ProblemParameter);
        //dlo.LoadWith<ProblemParameterBoolValue>(x => x.DataType);
        //dlo.LoadWith<ProblemParameterFloatValue>(x => x.ProblemParameter);
        //dlo.LoadWith<ProblemParameterFloatValue>(x => x.DataType);
        //dlo.LoadWith<ProblemParameterIntValue>(x => x.ProblemParameter);
        //dlo.LoadWith<ProblemParameterIntValue>(x => x.DataType);
        //dlo.LoadWith<ProblemParameterStringValue>(x => x.ProblemParameter);
        //dlo.LoadWith<ProblemParameterStringValue>(x => x.DataType);
        //dlo.LoadWith<AlgorithmParameter>(x => x.Algorithm);
        //dlo.LoadWith<AlgorithmParameter>(x => x.DataType);
        //dlo.LoadWith<ProblemParameter>(x => x.Problem);
        //dlo.LoadWith<ProblemParameter>(x => x.DataType);
        //dlo.LoadWith<Algorithm>(x => x.AlgorithmClass);
        //dlo.LoadWith<Algorithm>(x => x.Platform);
        //dlo.LoadWith<Problem>(x => x.ProblemClass);
        //dlo.LoadWith<Problem>(x => x.Platform);
        //okb.LoadOptions = dlo;

        DataLoadOptions dlo = new DataLoadOptions();
        dlo.LoadWith<Run>(x => x.ResultBlobValues);
        dlo.LoadWith<Run>(x => x.ResultBoolValues);
        dlo.LoadWith<Run>(x => x.ResultFloatValues);
        dlo.LoadWith<Run>(x => x.ResultIntValues);
        dlo.LoadWith<Run>(x => x.ResultStringValues);
        okb.LoadOptions = dlo;

        IFilter convertedFilter = (IFilter)Activator.CreateInstance(Type.GetType(filter.FilterTypeName), filter);
        var runs = okb.Runs.Where(convertedFilter.Expression).ToArray();
        var result = runs.Select(x => Convert.ToDto(x)).ToArray();
        return result;
      }
    }
  }
}
